/* ---------------------------------------------------------------------------
   This test-set checks the behavior of some int64 related built-in  functions
   in normal conditions and when Progress is started with  -noint64  argument.
   Even if the documentation of Progress 4GL states that these functions  will
   return integer instead of int64 when  running  with  -noint64  command-line
   parameter, this is not proved in practice as they can  return  values  that
   pass the 32-bit of integer type: 
   * INT64 is available;
   * sequence statements/functions seem not to be affected;
   * ETIME / INTERVAL functions seem not to be affected;
   * GET-SIZE / SET-SIZE / SEEK / GET-POINTER-VALUE will  only  accept  32-bit
         values in any case, probably because of the Progress internal  32-bit
         memory architecture.

   The only observable differences are that in -noint64 mode:
   * int64 fields and variables can be defined BUT not accessed;
   * you cannot substract two datetime values, you must use INTERVAL  built-in
         function with "millisecond" as last parameter. Adding an int64 literal 
         to a datetime variable is also not possible (because they are seen as
         decimals), BUT using expressions (like wrapping the literal in a int64
         function) will permit the addition operation.
   --------------------------------------------------------------------------- */

DEFINE VARIABLE m AS MEMPTR NO-UNDO.
SET-SIZE(m) = 200000.
DISPLAY "Allocate memptr: " GET-SIZE(m) SKIP.

SET-SIZE(m) = 0.
DISPLAY "Resetted memptr: " GET-SIZE(m) SKIP. 

SET-SIZE(m) = 400000. /* 2147483647: seems like P4GL only support 32 bit setter here regardless 
                           of -noint64 parameter. However, probably memory resources restrict 
                           this parameter evem more. On the other side, on P2J the size of the 
                           variable is also limited by java heap limit! */
DISPLAY "Realloc memptr:  " GET-SIZE(m) SKIP.

CURRENT-VALUE(my-seq-1) = 2147483647 * 2147483647.
DYNAMIC-CURRENT-VALUE("my-seq-1", "p2j_test") = 2147483647 * 2147483647 + 2147483647.

DISPLAY "ETIME:                         " ETIME FORMAT "->>>>>>>>>>>>>>>>>>>9" SKIP.
DISPLAY "CURRENT-VALUE(my-seq-1):       " CURRENT-VALUE(my-seq-1) FORMAT "->>>>>>>>>>>>>>>>>>>9" SKIP.
DISPLAY "DYNAMIC-CURRENT-VALUE(my-seq-1)" DYNAMIC-CURRENT-VALUE("my-seq-1", "p2j_test")  FORMAT "->>>>>>>>>>>>>>>>>>>9" SKIP .
DISPLAY "DYNAMIC-NEXT-VALUE(my-seq-1)   " DYNAMIC-NEXT-VALUE("my-seq-1", "p2j_test")  FORMAT "->>>>>>>>>>>>>>>>>>>9" SKIP .
DISPLAY "INTERVAL since DATE(0)         " INTERVAL(NOW, DATETIME(DATE(0)), "milliseconds") FORMAT "->>>>>>>>>>>>>>>>>>>9" SKIP.
DISPLAY "DYNAMIC-NEXT-VALUE(my-seq-1)   " NEXT-VALUE(my-seq-1) FORMAT "->>>>>>>>>>>>>>>>>>>9" SKIP.

DISPLAY "DATETIME + number expression   " NOW + int64(3000000000000) SKIP .
/* DISPLAY "DATETIME difference:           " NOW - DATETIME(DATE(0)) FORMAT "->>>>>>>>>>>>>>>>>>>9" SKIP.
                                    -- this is not compilable on -noint64 mode */

DEFINE VARIABLE oioioi AS INT64.    /* int64 variables can be defined in -noint64 mode */
/* DISPLAY oioioi.                  -- but they cannot be accessed */
/* oioioi = 10.                     -- but they cannot be accessed */

/* Expected output:
┌─────────────────────────────────────────────────────────────┐
│Allocate memptr:     200,000                                 │
│Resetted memptr:           0                                 │
│Realloc memptr:      400,000                                 │
│ETIME:                                       25460343        │
│CURRENT-VALUE(my-seq-1):          4611686016279904256        │
│DYNAMIC-CURRENT-VALUE(my-seq-1)   4611686016279904256        │
│DYNAMIC-NEXT-VALUE(my-seq-1)      4611686016279904259        │
│INTERVAL since DATE(0)                212233012738167        │
│DYNAMIC-NEXT-VALUE(my-seq-1)      4611686016279904262        │
│DATETIME + number expression    18/10/2022 19:58:58.169+01:00│
└─────────────────────────────────────────────────────────────┘
*/